#include <math.h>
#include "nist.h"

/*
	Function declarations from EOSPK that are used in this file:
*/
void AIDEAL(int LPHI, int LPHID, int LPHIT, int LPHIDD, int LPHITT, int LPHIDT,
            int LPHDDD, int LPHDDT, int LPHDTT, double *PHI, double *PHID, double *PHIT,
            double *PHIDD, double *PHITT, double *PHIDT, double *PHIDDD, double *PHIDDT, 
            double *PHIDTT, double TAU, double DEL);
void ARESID(int LPHI, int LPHID, int LPHIT, int LPHIDD, int LPHITT, int LPHIDT,
            int LPHDDD, int LPHDDT, int LPHDTT, double *PHI, double *PHID, double *PHIT,
            double *PHIDD, double *PHITT, double *PHIDT, double *PHIDDD, double *PHIDDT, 
            double *PHIDTT, double TAU, double DEL);


/*
	Function declarations used in this file:
*/
void PROPS2(int IWANT[], int IGFLG, double TR, double DEL, double PROPR[]);
void WTRANS(int MODE, double TK, double RHO, double CHI, double DPDT, double *THCOND, 
            double *VISCOS);
int DIELEC(double TK, double RHO, double *DCON, double *DEPSDT, double *DEPSDR, 
           double *D2EDTT, double *D2EDRR, double *D2EDTR, int MODE);
void VISC(double TBAR, double RBAR, double CHIBAR, double *VISC0, double *VISCBR);
double BKCOND(double TBAR, double RBAR);
double ANCOND(double TBAR, double RBAR, double CHIBAR, double DPDTBR, double VBAR0);
double RIND2(double ALAMBR, double RBAR, double TBAR);


/*
	Function definitions:
*/
void KFACT(double TR, double RHOL, double RHOV, double *FLIQ, double *FVAP, double *XK, 
           int IWANT[], double PROPR[])
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** KFACT ***                                    C
C THIS ROUTINE COMPUTES THE K-FACTOR AS THE RATIO OF FUGACITY          C
C COEFFICIENTS AT A GIVEN REDUCED TEMPERATURE TR AND LIQUID AND VAPOR  C
C DENSITIES RHOL AND RHOV.                                             C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C TR      R    I   REDUCED TEMPERATURE, T/TC                           C
C RHOL    R    I   REDUCED LIQUID DENSITY, RHOL/RHOC                   C
C RHOV    R    I   REDUCED VAPOR DENSITY, RHOV/RHOC                    C
C FLIQ    R    O   LIQUID FUGACITY COEFFICIENT                         C
C FVAP    R    O   VAPOR FUGACITY COEFFICIENT                          C
C XK      R    O   K-FACTOR = FLIQ/FVAP                                C
C IWANT   IA   -   PROPERTY REQUEST VECTOR FOR USE IN PROPS2 ROUTINE   C
C PROPR   RA   O   VECTOR OF REDUCED PROPERTIES AS REQUESTED BY IWANT  C
C                  TO BE RETURNED BY THE PROP2 ROUTINE                 C
C                  RELEVANT PROPERTIES HERE ARE:                       C
C                 13: FUGACITY COEFFICIENT                             C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 20JUL95 - INITIAL CREATION BY AHH                                    C
C 05SEP95 - AHH: CHANGE TO TR IN PROPS2 ARGUMENT LIST                  C
C 28SEP95 - AHH: PARAMETERIZE NUMBER OF PROPERTIES                     C
C 11DEC95 - AHH: RENUMBERING OF PROPERTY ARRAY                         C
C 30AUG96 - AHH: LOWERCASE INCLUDES FOR OTHER PLATFORMS                C
C 08JAN97 - AHH: CHANGE IN ARGUMENT LIST OF PROPS2                     C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SET IWANT VECTOR TO RETURN FUGACITY COEFFICIENT
*/
    IVZERO(IWANT, NPROP);
    IWANT[13] = 1;
/*
C CALL PROPS2 TO RETURN VAP AND LIQ FUGACITY COEFFICIENTS
*/
    PROPS2(IWANT, 0, TR, RHOL, PROPR);
    *FLIQ = PROPR[13];
    
    PROPS2(IWANT, 0, TR, RHOV, PROPR);
    *FVAP = PROPR[13];

    *XK = *FLIQ / *FVAP;

    return;
}


void PDP(double DEL, double TR, double *PR, double *DPRDD, int IWANT[], double PROPR[])
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** PDP ***                                      C
C THIS ROUTINE COMPUTES THE REDUCED PRESSURE P/PC AND ITS FIRST        C
C DERIVATIVE WITH RESPECT TO REDUCED DENSITY AT A GIVEN REDUCED        C
C TEMPERATURE TR AND REDUCED DENSITY DEL.                              C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C DEL     R    I   REDUCED DENSITY, RHO/RHOC                           C
C TR      R    I   REDUCED TEMPERATURE, T/TC                           C
C PR      R    O   REDUCED PRESSURE, P/PC                              C
C DPRDD   R    O   FIRST DERIVATIVE OF PR WITH RESPECT TO DEL          C
C IWANT   IA   -   PROPERTY REQUEST VECTOR FOR USE IN PROPS2 ROUTINE   C
C PROPR   RA   O   VECTOR OF REDUCED PROPERTIES AS REQUESTED BY IWANT  C
C                  TO BE RETURNED BY THE PROP2 ROUTINE                 C
C                  RELEVANT PROPERTIES HERE ARE:                       C
C                  2: REDUCED PRESSURE P/(RHO*R*T)                     C
C                 16: REDUCED DP/DRHO DPDR/RT                          C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 12JUL95 - INITIAL CREATION BY AHH                                    C
C 20JUL95 - AHH: PROPR AND IWANT NOW DIMENSIONED TO 19                 C
C 05SEP95 - AHH: CHANGE TO TR IN PROPS2 ARGUMENT LIST                  C
C 28SEP95 - AHH: PARAMETERIZE NUMBER OF PROPERTIES                     C
C 11DEC95 - AHH: RENUMBERING OF PROPERTY VECTOR                        C
C 30AUG96 - AHH: LOWERCASE INCLUDES FOR OTHER PLATFORMS                C
C 08JAN97 - AHH: CHANGE IN ARGUMENT LIST OF PROPS2                     C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
	double RHO, RTW;
/*
C SET IWANT VECTOR TO RETURN P AND DPDD
*/
    IVZERO(IWANT, NPROP);
    IWANT[2] = 1;
    IWANT[16] = 1;
/*
C CALL PROPS2 TO RETURN DIMENSIONLESS PROPERTIES
*/
    PROPS2(IWANT, 0, TR, DEL, PROPR);
/*
C THIS RETURNED PRESSURES REDUCED BY RHO*RT, SO MUST CONVERT TO PC
C AS REDUCING BASIS
*/
    RHO = DEL*RHOCW;
    RTW = RW*TR*TCW;
    *PR = PROPR[2]*RHO*RTW*1.0e-3/PCW;
    *DPRDD = PROPR[16]*RTW*1.0e-3/PCW*RHOCW;
    
    return;
}


void PROPS1(int IWANT[], int IGFLG, double TK, double RHO, double PROPSI[], double PROPR[])
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** PROPS1 ***                                   C
C THIS ROUTINE COMPUTES ALL PROPERTIES REQUESTED VIA THE IWANT VECTOR  C
C AT A GIVEN TEMPERATURE AND DENSITY.  PROPERTIES ARE RETURNED IN SI   C
C UNITS: KG, M3, K, MPA, KJ.  THE LOWER-LEVEL ROUTINE PROPS2 IS        C
C CALLED TO RETURN THE PROPERTIES IN DIMENSIONLESS FORM. WTRANS IS     C
C CALLED TO RETURN THE VISCOSITY AND THERMAL CONDUCTIVITY. DIELEC IS   C
C CALLED TO RETURN THE DIELECTRIC CONSTANT AND ITS DERIVATIVES         C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C IWANT   IA   I   FOR WHETHER OR NOT TO COMPUTE VARIOUS PROPERTIES,   C
C                  SEE PROPR ARRAY BELOW FOR NUMBERING OF PROPERTIES.  C
C                  1=COMPUTE, 0=DON'T COMPUTE                          C
C IGFLG   I    I   WHETHER TO COMPUTE AND RETURN ONLY IDEAL-GAS PROPS  C
C                  1=IDEAL GAS, 0=REAL FLUID                           C
C TK      R    I   TEMPERATURE IN KELVINS                              C
C RHO     R    I   DENSITY IN KG/M3                                    C
C PROPSI  RA   O   VECTOR OF PROPERTIES IN SI UNITS AS REQUESTED BY    C
C                  IWANT.  NUMBERING OF PROPERTIES:                    C
C                  1: TEMPERATURE IN K                                 C
C                  2: PRESSURE IN MPA                                  C
C                  3: DENSITY IN KG/M3                                 C
C                  4: VOLUME IN M3/KG                                  C
C                  5: QUALITY (ONLY SET IF 2-PHASE)                    C
C                  6: ENTHALPY IN KJ/KG                                C
C                  7: ENTROPY IN KJ/(KG*K)                             C
C                  8: ISOCHORIC HEAT CAPACITY IN KJ/(KG*K)             C
C                  9: ISOBARIC HEAT CAPACITY IN KJ/(KG*K)              C
C                 10: INTERNAL ENERGY IN KJ/KG                         C
C                 11: HELMHOLTZ ENERGY IN KJ/KG                        C
C                 12: GIBBS ENERGY IN KJ/KG                            C
C                 13: FUGACITY IN MPA                                  C
C                 14: ISOTHERMAL COMPRESSIBILITY IN MPA**(-1)          C
C                 15: VOLUME EXPANSIVITY IN K**(-1)                    C
C                 16: DP/DRHO (CONSTANT T) IN MPA/(KG/M3)              C
C                 17: DP/DT (CONSTANT RHO) IN MPA/K                    C
C                 18: SPEED OF SOUND IN M/SEC                          C
C                 19: JOULE-THOMSON COEFFICIENT IN K/MPA               C
C                 20: THERMAL CONDUCTIVITY IN W/M-K                    C
C                 21: VISCOSITY IN PA-SEC                              C
C                 22: STATIC DIELECTRIC CONSTANT (RETURNED AS -999. IF C
C                     CALCULATION COULD NOT BE DONE)                   C
C                 23: 2ND DERIVATIVE D2P/DRHO2 (CONSTANT T) IN         C
C                     MPA/(KG/M3)**2                                   C
C                 24: 2ND DERIVATIVE D2P/DT2 (CONSTANT RHO)            C
C                     IN MPA/K**2                                      C
C                 25: 2ND DERIVATIVE D2P/DRHODT (CONSTANT T, RHO)      C
C                     IN MPA/(KG/M3)/K                                 C
C                 26: DRHO/DT (CONSTANT P) IN (KG/M3)/K                C
C                 27: DRHO/DP (CONSTANT T) IN (KG/M3)/MPA              C
C                 28: 2ND DERIVATIVE D2RHO/DT2 (CONSTANT P)            C
C                     IN (KG/M3)/K**2                                  C
C                 29: 2ND DERIVATIVE D2RHO/DP2 (CONSTANT T)            C
C                     IN (KG/M3)/MPA**2                                C
C                 30: 2ND DERIVATIVE D2RHO/DTDP (CONSTANT P, T)        C
C                     IN (KG/M3)/K/MPA                                 C
C                 31: DIELECTRIC DERIVATIVE D(EPS)/D(RHO) (CONSTANT T) C
C                     IN M3/KG                                         C
C                 32: DIELECTRIC DERIVATIVE D(EPS)/DT (CONSTANT RHO)   C
C                     IN K**(-1)                                       C
C                 33: DIELECTRIC DERIVATIVE D(EPS)/DT (CONSTANT P)     C
C                     IN K**(-1)                                       C
C                 34: DIELECTRIC DERIVATIVE D(EPS)/DP (CONSTANT T)     C
C                     IN MPA**(-1)                                     C
C                 35: 2ND DIELECTRIC DERIVATIVE D2(EPS)/DT2 (CONST. P) C
C                     IN K**(-2)                                       C
C                 36: 2ND DIELECTRIC DERIVATIVE D2(EPS)/DP2 (CONST. T) C
C                     IN MPA**(-2)                                     C
C                 37: 2ND DIELECTRIC DERIVATIVE D2(EPS)/DTDP           C
C                     (CONSTANT P, T) IN (K*MPA)**(-1)                 C
C                 38: DEBYE-HUCKEL SLOPE FOR ACTIVITY COEFFICIENT      C
C                     IN (KG/MOL)**0.5                                 C
C                 39: D-H SLOPE FOR OSMOTIC COEFF. IN (KG/MOL)**0.5    C
C                 40: D-H SLOPE FOR APPARENT MOLAR VOLUME              C
C                     IN (M3/MOL)*(KG/MOL)**0.5                        C
C                 41: D-H SLOPE FOR APPARENT MOLAR ENTHALPY, DIVIDED   C
C                     BY RT, IN (KG/MOL)**0.5                          C
C                 42: D-H SLOPE FOR APPARENT MOLAR COMPRESSIBILITY     C
C                     IN (M3/MOL/MPA)*(KG/MOL)**0.5                    C
C                 43: D-H SLOPE FOR APPARENT MOLAR HEAT CAPACITY,      C
C                     DIVIDED BY R, IN (KG/MOL)**0.5                   C
C PROPR   RA   O   VECTOR OF REDUCED PROPERTIES AS REQUESTED BY IWANT  C
C                  TO BE RETURNED BY THE PROPS2 ROUTINE                C
C                  1: TEMPERATURE (SLOT NOT USED)                      C
C                  2: REDUCED PRESSURE P/(RHO*R*T)                     C
C                  3: DENSITY (SLOT NOT USED)                          C
C                  4: VOLUME (SLOT NOT USED)                           C
C                  5: QUALITY (SLOT NOT USED)                          C
C                  6: REDUCED ENTHALPY H/RT                            C
C                  7: REDUCED ENTROPY S/R                              C
C                  8: REDUCED ISOCHORIC HEAT CAPACITY CV/R             C
C                  9: REDUCED ISOBARIC HEAT CAPACITY CP/R              C
C                 10: REDUCED INTERNAL ENERGY U/RT                     C
C                 11: REDUCED HELMHOLTZ ENERGY A/RT                    C
C                 12: REDUCED GIBBS ENERGY G/RT                        C
C                 13: FUGACITY COEFFICIENT FUG/PRES                    C
C                 14: REDUCED ISOTHERMAL COMPRESSIBILITY (COMPR*RHO*RT,C
C                     RT/DPDR)                                         C
C                 15: REDUCED VOLUME EXPANSIVITY EXPANS*T              C
C                 16: REDUCED DP/DRHO (CONSTANT T) DPDR/RT             C
C                 17: REDUCED DP/DT (CONSTANT RHO) DPDT/(RHO*R)        C
C                 18: REDUCED SPEED OF SOUND (W**2)/RT                 C
C                 19: REDUCED JOULE-THOMSON COEFFICIENT MU*R*RHO       C
C                 20: THERMAL CONDUCTIVITY (SLOT NOT USED)             C
C                 21: VISCOSITY (SLOT NOT USED)                        C
C                 22: DIELECTRIC CONSTANT (SLOT NOT USED)              C
C                 23: REDUCED 2ND DERIVATIVE D2P/DRHO2 (CONSTANT T)    C
C                     D2PDRR*RHO/RT                                    C
C                 24: REDUCED 2ND DERIVATIVE D2P/DT2 (CONSTANT RHO)    C
C                     D2PDTT*T/(RHO*R)                                 C
C                 25: REDUCED 2ND DERIVATIVE D2P/DRHODT (CONSTANT T,   C
C                     RHO) D2PDRT/R                                    C
C                 26: REDUCED DRHO/DT (CONSTANT P) DRDT*T/RHO          C
C                 27: REDUCED DRHO/DP (CONSTANT T) DRDP*RT             C
C                 28: REDUCED 2ND DERIVATIVE D2RHO/DT2 (CONSTANT P)    C
C                     D2RDTT*T*T/RHO                                   C
C                 29: REDUCED 2ND DERIVATIVE D2RHO/DP2 (CONSTANT T)    C
C                     D2RDPP*RHO*(RT**2)                               C
C                 30: REDUCED 2ND DERIVATIVE D2RHO/DTDP (CONSTANT P,T) C
C                     D2RDTP*R*T*T                                     C
C                 31-37: DIELECTRIC DERIVATIVES (SLOTS NOT USED)       C
C                 38-43: DEBYE-HUCKEL SLOPES (SLOTS NOT USED)          C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 11JUL95 - INITIAL CREATION BY AHH                                    C
C 20JUL95 - AHH: ADDITION OF FUGACITY AS PROPERTY 19                   C
C 18AUG95 - VERSION AFTER DEBUGGING BY AHH AND JW                      C
C 05SEP95 - CHANGE TO TR IN PROPS2 ARGUMENT LIST                       C
C 11SEP95 - FILL IN TEMPERATURE AND DENSITY SLOTS IF REQUESTED         C
C 19SEP95 - FOR GUI, MAKE ANY IWANT .NE. 0  GIVE PROPERTY COMPUTATION  C
C 25SEP95 - AHH: PUT IN THERMAL CONDUCTIVITY AND VISCOSITY             C
C 28SEP95 - AHH: RESET IWANT FLAGS CHANGED INTERNALLY                  C
C 28SEP95 - AHH: PARAMETERIZE NUMBER OF PROPERTIES                     C
C 27NOV95 - AHH: ADD DIELECTRIC CONSTANT IN SLOT 21                    C
C 07DEC95 - AHH: ADD VOLUME EXPANSIVITY AS PROPERTY 22                 C
C 11DEC95 - AHH: REORDER NUMBERING IN PROPERTY VECTORS                 C
C 12DEC95 - AHH: CORRECT EXPANSIVITY CALCULATION                       C
C 15DEC95 - AHH: ALLOW FOR FAILURE OF DIELECTRIC CALCULATION           C
C 15FEB96 - AHH: CHANGE ARGUMENT LIST FOR DIELEC SUBROUTINE            C
C 30AUG96 - AHH: LOWERCASE INCLUDES FOR OTHER PLATFORMS                C
C 17SEP96 - AHH: EXPUNGE TAB CHARACTERS                                C
C 08JAN97 - AHH: ADD FLAG FOR COMPUTING IDEAL-GAS PROPERTIES           C
C 09JAN97 - AHH: FILL IN OTHER PROPS (DIELEC, TRANS) FOR IDEAL GAS     C
C 09JAN97 - AHH: CHANGE ARGUMENT LIST FOR DIELEC                       C
C 11FEB97 - AHH: MAKE SQRT UNIFORMLY DOUBLE PRECISION                  C
C 26FEB97 - AHH: ADD DERIVATIVE PROPERTIES IN SLOTS 23-30              C
C 05MAR97 - AHH: ADD DIELECTRIC DERIVATIVES IN SLOTS 31-37             C
C 06MAR97 - AHH: ADD DEBYE-HUCKEL SLOPES IN SLOTS 38-43                C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
	static const double ECH = 1.60217733e-19; 
	static const double BOLTZ = 1.380658e-23;
    static const double AV = 6.0221367e23;
    
    double TR, DEL, RTW, D2EDTT, D2EDRR, D2EDTR, DEPTR; 
    double THING1, THING2, PI, EPS0, RGAS, C1, C2, C3, EPS;
    
    int ISAV14, ISAV17, ISAV22, ISAV23, ISAV24, ISAV25, ISAV26, ISAV27, ISAV28, ISAV29, ISAV31;
    int ISAV33, ISAV34, ISAV35, ISAV36, ISAV37, ISAV39, ISAV40, ISAV41, MODE, MODEDH, MODED, IERD;
/*
C COMPUTE REDUCED TEMPERATURE AND REDUCED DENSITY
*/
    TR  = TK/TCW;
    DEL = RHO/RHOCW;
    RTW = RW*TK;
/*
C SAVE SOME IWANT FLAGS THAT MIGHT GET CHANGED WITHIN THE ROUTINE
*/
    ISAV14 = IWANT[14];
    ISAV17 = IWANT[17];
    ISAV22 = IWANT[22];
    ISAV23 = IWANT[23];
    ISAV24 = IWANT[24];
    ISAV25 = IWANT[25];
    ISAV26 = IWANT[26];
    ISAV27 = IWANT[27];
    ISAV28 = IWANT[28];
    ISAV29 = IWANT[29];
    ISAV31 = IWANT[31];
    ISAV33 = IWANT[33];
    ISAV34 = IWANT[34];
    ISAV35 = IWANT[35];
    ISAV36 = IWANT[36];
    ISAV37 = IWANT[37];
    ISAV39 = IWANT[39];
    ISAV40 = IWANT[40];
    ISAV41 = IWANT[41];
/*
C IF THERMAL CONDUCTIVITY OR VISCOSITY REQUESTED, COMPUTE OTHER 
C PROPERTIES NEEDED FOR THOSE COMPUTATIONS
*/
    MODE = 0;
      
    if (IWANT[20] > 0) 
    {
        IWANT[17] = 1;
        IWANT[14] = 1;
        MODE = MODE + 1;
    }
      
    if (IWANT[21] > 0) 
    {
        IWANT[14] = 1;
        MODE = MODE + 2;
    }
/*
C FOR SOME DEBYE-HUCKEL SLOPES, NEED TO COMPUTE CERTAIN DERIVATIVES
*/
    MODEDH = 1;
      
    if (IWANT[43] != 0) 
    {
        IWANT[41] = 1;
        IWANT[35] = 1;
        IWANT[28] = 1;
    }
      
    if (IWANT[42] != 0) 
    {
        IWANT[40] = 1;
        IWANT[36] = 1;
        IWANT[29] = 1;
    }
      
    if (IWANT[41] != 0) 
    {
        IWANT[39] = 1;
        IWANT[33] = 1;
        IWANT[26] = 1;
    }
      
    if (IWANT[40] != 0) 
    {
        IWANT[39] = 1;
        IWANT[34] = 1;
        IWANT[27] = 1;
    }
      
    if ((IWANT[39] != 0) || (IWANT[38] != 0)) 
    {
        IWANT[22] = 1;
        MODEDH = 1;
    }
/*
C FOR SOME DIELECTRIC DERIVATIVES, ALSO NEED SOME OTHER PROPERTIES
*/
    MODED = 0;
    
    if (IWANT[35] > 0) 
    {
        IWANT[24] = 1;
        IWANT[37] = 1;
        MODED = 2;
    }
      
    if (IWANT[36] > 0) 
    {
        IWANT[23] = 1;
        IWANT[27] = 1;
        IWANT[31] = 1;
        MODED = 2;
    }
      
    if (IWANT[37] > 0) 
    {
        IWANT[17] = 1;
        IWANT[23] = 1;
        IWANT[25] = 1;
        IWANT[31] = 1;
        IWANT[34] = 1;
        MODED = 2;
    }
      
    if ((IWANT[31] != 0) || (IWANT[32] != 0) || (IWANT[33] != 0) || (IWANT[34] != 0)) 
        MODED = (MODED > 1) ? MODED : 1;
      
    if (IWANT[33] > 0) 
    {
        IWANT[17] = 1;
        IWANT[27] = 1;
    }
      
    if (IWANT[34] > 0) 
        IWANT[27] = 1;
    
/*
C CALL PROPS2 TO RETURN DIMENSIONLESS PROPERTIES
*/
    PROPS2(IWANT, IGFLG, TR, DEL, PROPR);
/*
C NOW FIGURE THE REQUESTED PROPERTIES IN SI UNITS
*/
    if (IWANT[1] != 0) 
        PROPSI[1] = TK;
      
    if (IWANT[2] != 0) 
        PROPSI[2] = PROPR[2]*RTW*RHO*1.0e-3;
      
    if (IWANT[3] != 0) 
        PROPSI[3] = RHO;
      
    if (IWANT[4] != 0) 
        PROPSI[4] = 1.0/RHO;
      
    if (IWANT[6] != 0) 
        PROPSI[6] = PROPR[6]*RTW;
      
    if (IWANT[7] != 0) 
        PROPSI[7] = PROPR[7]*RW;
      
    if (IWANT[8] != 0) 
        PROPSI[8] = PROPR[8]*RW;
      
    if (IWANT[9] != 0) 
        PROPSI[9] = PROPR[9]*RW;
      
    if (IWANT[10] != 0) 
        PROPSI[10] = PROPR[10]*RTW;
      
    if (IWANT[11] != 0) 
        PROPSI[11] = PROPR[11]*RTW;
      
    if (IWANT[12] != 0) 
        PROPSI[12] = PROPR[12]*RTW;
      
    if (IWANT[13] != 0) 
    {
        PROPSI[2] = PROPR[2]*RTW*RHO*1.0e-3;
        PROPSI[13] = PROPR[13]*PROPSI[2];
    }
      
    if (IWANT[14] != 0) 
        PROPSI[14] = PROPR[14]/RHO/RTW*1.0e3;
      
    if (IWANT[15] != 0) 
        PROPSI[15] = PROPR[15]/TK;
      
    if (IWANT[16] != 0) 
        PROPSI[16] = PROPR[16]*RTW*1.0e-3;
      
    if (IWANT[17] != 0) 
        PROPSI[17] = PROPR[17]*RHO*RW*1.0e-3;
      
    if (IWANT[18] != 0) 
    {
        if (PROPR[18] > 0.0) 
          PROPSI[18] = sqrt(PROPR[18]*RTW*1.0e3);
        else
          PROPSI[18] = 0.0;
    }
    
    if (IWANT[19] != 0) 
        PROPSI[19] = PROPR[19]/RHO/RW*1.0e3;
              
    if (IWANT[23] != 0) 
        PROPSI[23] = PROPR[23]*RTW/RHO * 1.0e-3;
      
    if (IWANT[24] != 0) 
        PROPSI[24] = PROPR[24]*RHO*RW/TK * 1.0e-3;
      
    if (IWANT[25] != 0) 
        PROPSI[25] = PROPR[25]*RW * 1.0e-3;
      
    if (IWANT[26] != 0) 
        PROPSI[26] = PROPR[26]*RHO/TK;
      
    if (IWANT[27] != 0) 
        PROPSI[27] = PROPR[27]/RTW * 1.0e3;
      
    if (IWANT[28] != 0) 
        PROPSI[28] = PROPR[28]*RHO/(TK*TK);
      
    if (IWANT[29] != 0) 
        PROPSI[29] = PROPR[29]/(RTW*RTW*RHO) * 1.0e6;
      
    if (IWANT[30] != 0) 
        PROPSI[30] = PROPR[30]/(RTW*TK) * 1.0e3;
      
    if ((IWANT[22] != 0) || (MODED > 0)) 
    {
        if (IGFLG == 1) 
        {
          	PROPSI[22] = 1.0;
          
          	if (MODED > 0) 
          	{
            	for (int i=31; i<=37; i++)
              		PROPSI[i] = 0.0;
          	}
        }
        else
        {
		/*
		C FIGURE OUT WHICH DIELECTRIC DERIVATIVES ARE NEEDED
		*/
          	IERD = DIELEC(TK, RHO, &PROPSI[22], &PROPSI[32], &PROPSI[31], &D2EDTT, &D2EDRR, 
          	              &D2EDTR, MODED);
          
          	if (IERD == 1)
          	{
            	PROPSI[22] = -999.0;
            
            	if (MODED > 0) 
            	{
            		for (int i=31; i<=37; i++)
              			PROPSI[i] = 0.0;
            	}
        	}
        	else
        	{	
            	if (IWANT[33] != 0) 
              		PROPSI[33] = PROPSI[32] - PROPSI[31]*PROPSI[17]*PROPSI[27];
            
            	if (IWANT[34] != 0) 
              		PROPSI[34] = PROPSI[31] * PROPSI[27];
            
            	if ((IWANT[36] != 0) || (IWANT[37] != 0)) 
            	{
              		DEPTR = PROPSI[27] * (D2EDRR - PROPSI[31]*PROPSI[27]*PROPSI[23]);
            
            		if (IWANT[36] != 0) 
                		PROPSI[36] = PROPSI[27] * DEPTR;
              
            		if (IWANT[37] != 0) 
                		PROPSI[37] = PROPSI[27] * (D2EDTR - DEPTR*PROPSI[17] - PROPSI[34]*PROPSI[25]);
        		}
            
        		if (IWANT[35] != 0) 
        		{
              		THING1 = PROPSI[27] * (D2EDTR - PROPSI[31]*PROPSI[27]*PROPSI[25]);
              		THING2 = D2EDTT - THING1*PROPSI[17] - PROPSI[34]*PROPSI[24];
              		PROPSI[35] = THING2 - PROPSI[37]*PROPSI[17];
        		}
        	}
        }
    }
/*
C FIGURE DEBYE-HUCKEL SLOPES
*/
    if (MODEDH == 1) 
    {
        PI = M_PI;		/*  C's way of defininf pi.  */
        EPS0 = 1.0 / (4.0e-7*PI*pow(299792458.0,2));
        RGAS = AV*BOLTZ;
/*
C - NOTE: WILL WRITE A SUB GAMMA AS C1*RHO**0.5 * C2*(EPS*T)**-1.5
C -       THEN A SUB PHI IS (1./3.)*C1*C2*(RHO**0.5)*(EPS*T)**1.5
*/
        C1 = 2.0*PI*AV;
        C2 = ECH*ECH / (4.0*PI*EPS0*BOLTZ);
        C3 = C2*sqrt(C1*C2);
        EPS = PROPSI[22];
        
        if (IWANT[38] != 0) 
          	PROPSI[38] = C3*sqrt(RHO)*pow(EPS*TK,-1.5);
        
        if (IWANT[39] != 0) 
          	PROPSI[39] = (C3*sqrt(RHO)*pow(EPS*TK,-1.5)) / 3.0;
        
        if (IWANT[40] != 0) 
          	PROPSI[40] = 2.0*RGAS*TK*1.0e-6*PROPSI[39] * (3.0*PROPSI[34]/EPS - PROPSI[27]/RHO);
        
        if (IWANT[41] != 0) 
          	PROPSI[41] = -6.0*PROPSI[39] * (1.0 + TK*PROPSI[33]/EPS - TK*PROPSI[26]/RHO/3.0);
        
        if (IWANT[42] != 0) 
          	PROPSI[42] = -0.5*PROPSI[40] * (3.0*PROPSI[34]/EPS - PROPSI[27]/RHO) +
						 2.0*RGAS*TK*1.0e-6*PROPSI[39] * (-3.0*pow(PROPSI[34]/EPS,2) + 
						 3.0*PROPSI[36]/EPS + pow(PROPSI[27]/RHO,2) - PROPSI[29]/RHO);
        
        if (IWANT[43] != 0) 
        {
          	THING1 = -1.50*PROPSI[41] * (1.0 + TK*PROPSI[33]/EPS - TK*PROPSI[26]/RHO/3.0);
          	
          	THING2 = -6.0*TK*PROPSI[39] * (1.0/TK + 2.0*PROPSI[33]/EPS - TK*pow(PROPSI[33]/EPS,2)
          	       + TK*PROPSI[35]/EPS + (-2.0*PROPSI[26]/RHO + TK*pow(PROPSI[26]/RHO,2)
    			   - TK*PROPSI[28]/RHO)/3.0);
     
          	PROPSI[43] = THING1 + THING2;
        }
    }
/*
C NOW CALL FOR TRANSPORT PROPERTIES IF REQUESTED
C NOTE THAT THE IDEAL-GAS VALUES FOR THESE ARE NOT MEANINGFUL,
C SO SET TO -999. IN THAT CASE.
*/
    if (MODE > 0) 
    {
    	if (IGFLG == 1) 
    	{
          	PROPSI[20] = -999.0;
          	PROPSI[21] = -999.0;
        }
        else
        {
          	WTRANS(MODE, TK, RHO, PROPSI[14], PROPSI[17], &PROPSI[20], &PROPSI[21]);
        }
    }
/*
C RESET ALL SAVED FLAGS
*/
    IWANT[14] = ISAV14;
    IWANT[17] = ISAV17;
    IWANT[22] = ISAV22;
    IWANT[23] = ISAV23;
    IWANT[24] = ISAV24;
    IWANT[25] = ISAV25;
    IWANT[26] = ISAV26;
    IWANT[27] = ISAV27;
    IWANT[28] = ISAV28;
    IWANT[29] = ISAV29;
    IWANT[31] = ISAV31;
    IWANT[33] = ISAV33;
    IWANT[34] = ISAV34;
    IWANT[35] = ISAV35;
    IWANT[36] = ISAV36;
    IWANT[37] = ISAV37;
    IWANT[39] = ISAV39;
    IWANT[40] = ISAV40;
    IWANT[41] = ISAV41;

    return;
}


void PROPS2(int IWANT[], int IGFLG, double TR, double DEL, double PROPR[])
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** PROPS2 ***                                   C
C THIS ROUTINE COMPUTES ALL REDUCED THERMODYNAMIC PROPERTIES REQUESTED C
C VIA THE IWANT VECTOR ACCORDING TO THE PRUSS-WAGNER EOS FOR STEAM.    C
C AT A GIVEN REDUCED TEMPERATURE AND DENSITY                           C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C IWANT   IA   I   FOR WHETHER OR NOT TO COMPUTE VARIOUS PROPERTIES,   C
C                  SEE PROPR ARRAY BELOW FOR NUMBERING OF PROPERTIES.  C
C                  1=COMPUTE, 0=DON'T COMPUTE                          C
C IGFLG   I    I   WHETHER TO COMPUTE AND RETURN ONLY IDEAL-GAS PROPS  C
C                  1=IDEAL GAS, 0=REAL FLUID                           C
C TR      R    I   DIMENSIONLESS TEMPERATURE T/TC                      C
C DEL     R    I   DIMENSIONLESS DENSITY RHO/RHOC                      C
C PROPR   RA   O   VECTOR OF REDUCED PROPERTIES AS REQUESTED BY IWANT  C
C                  TO BE RETURNED BY THE PROPS2 ROUTINE                C
C                  1: TEMPERATURE (SLOT NOT USED)                      C
C                  2: REDUCED PRESSURE P/(RHO*R*T)                     C
C                  3: DENSITY (SLOT NOT USED)                          C
C                  4: VOLUME (SLOT NOT USED)                           C
C                  5: QUALITY (SLOT NOT USED)                          C
C                  6: REDUCED ENTHALPY H/RT                            C
C                  7: REDUCED ENTROPY S/R                              C
C                  8: REDUCED ISOCHORIC HEAT CAPACITY CV/R             C
C                  9: REDUCED ISOBARIC HEAT CAPACITY CP/R              C
C                 10: REDUCED INTERNAL ENERGY U/RT                     C
C                 11: REDUCED HELMHOLTZ ENERGY A/RT                    C
C                 12: REDUCED GIBBS ENERGY G/RT                        C
C                 13: FUGACITY COEFFICIENT FUG/PRES                    C
C                 14: REDUCED ISOTHERMAL COMPRESSIBILITY (COMPR*RHO*RT,C
C                     RT/DPDR)                                         C
C                 15: REDUCED VOLUME EXPANSIVITY EXPANS*T              C
C                 16: REDUCED DP/DRHO (CONSTANT T) DPDR/RT             C
C                 17: REDUCED DP/DT (CONSTANT RHO) DPDT/(RHO*R)        C
C                 18: REDUCED SPEED OF SOUND (W**2)/RT                 C
C                 19: REDUCED JOULE-THOMSON COEFFICIENT MU*R*RHO       C
C                 20: THERMAL CONDUCTIVITY (SLOT NOT USED)             C
C                 21: VISCOSITY (SLOT NOT USED)                        C
C                 22: DIELECTRIC CONSTANT (SLOT NOT USED)              C
C                 23: REDUCED 2ND DERIVATIVE D2P/DRHO2 (CONSTANT T)    C
C                     D2PDRR*RHO/RT                                    C
C                 24: REDUCED 2ND DERIVATIVE D2P/DT2 (CONSTANT RHO)    C
C                     D2PDTT*T/(RHO*R)                                 C
C                 25: REDUCED 2ND DERIVATIVE D2P/DRHODT (CONSTANT T,   C
C                     RHO) D2PDRT/R                                    C
C                 26: REDUCED DRHO/DT (CONSTANT P) DRDT*T/RHO          C
C                 27: REDUCED DRHO/DP (CONSTANT T) DRDP*RT             C
C                 28: REDUCED 2ND DERIVATIVE D2RHO/DT2 (CONSTANT P)    C
C                     D2RDTT*T*T/RHO                                   C
C                 29: REDUCED 2ND DERIVATIVE D2RHO/DP2 (CONSTANT T)    C
C                     D2RDPP*RHO*(RT**2)                               C
C                 30: REDUCED 2ND DERIVATIVE D2RHO/DTDP (CONSTANT P,T) C
C                     D2RDTP*R*T*T                                     C
C                 31-37: DIELECTRIC DERIVATIVES (SLOTS NOT USED)       C
C                 38-43: DEBYE-HUCKEL SLOPES (SLOTS NOT USED)          C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 11JUL95 - INITIAL CREATION BY AHH                                    C
C 20JUL95 - AHH: ADD FUGACITY COEFFICIENT IN SLOT 19                   C
C 18AUG95 - REVISED VERSION AFTER DEBUGGING BY AHH AND JW              C
C 21AUG95 - AHH: FIX FUGACITY COEFFICIENT COMPUTATION                  C
C 05SEP95 - AHH: USE TR INSTEAD OF TAU IN ARGUMENT LIST                C
C 11SEP95 - AHH: RETURN ZERO FUGACITY IF PRESSURE IS NEGATIVE          C
C 19SEP95 - FOR GUI, MAKE ANY IWANT .NE. 0  GIVE PROPERTY COMPUTATION  C
C 28SEP95 - AHH: PARAMETERIZE NUMBER OF PROPERTIES                     C
C 07DEC95 - AHH: ADD VOLUME EXPANSIVITY AS PROPERTY 22                 C
C 11DEC95 - AHH: REORDER NUMBERING OF PROPERTY VECTOR                  C
C 30AUG96 - AHH: LOWERCASE INCLUDES FOR OTHER PLATFORMS                C
C 08JAN97 - AHH: ADD FLAG FOR COMPUTING IDEAL-GAS PROPERTIES           C
C 17JAN97 - AHH: CHANGE ARGUMENT LISTS IN ARESID AND AIDEAL            C
C 11FEB97 - AHH: MAKE EXP'S UNIFORMLY DOUBLE PRECISION                 C
C 27FEB97 - AHH: ADD 8 SLOTS FOR DERIVATIVES (23-30)                   C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
	double TAU, PHI0, PHID0, PHIT0, PHIDD0, PHITT0, PHIDT0, PHDDD0, PHDDT0, PHDTT0;
	double PHIR, PHIDR, PHITR, PHIDDR, PHITTR, PHIDTR, PHDDDR, PHDDTR, PHDTTR;
	double C, B, DRT, DCR, DCT, DBT, DBR, DXT, DXR, DRP, D2PDRR;
/*  
	The following were logical variables in Fortran, in C they will be integers with
	a value of '0' specifying false, and non-zero equating to true.
*/
    int LPHI, LPHID, LPHIDD, LPHIT, LPHITT, LPHIDT;
    int LPHDDD, LPHDDT, LPHDTT;
/*
C INITIALIZE LOGICALS TO FALSE
*/
    LPHI = 0;
    LPHID = 0;
    LPHIDD = 0;
    LPHIT = 0;
    LPHITT = 0;
    LPHIDT = 0;
    LPHDDD = 0;
    LPHDDT = 0;
    LPHDTT = 0;

    TAU = 1.0/TR;
/*
C NOW LOOP THROUGH AND SET WHICH PARTS OF THE HELMHOLTZ ENERGY YOU HAVE
C TO COMPUTE BASED ON THE PROPERTIES REQUESTED
*/
    if (IWANT[2] != 0) 
        LPHID = 1;
      
    if (IWANT[6] != 0) 
    {
        LPHID = 1;
        LPHIT = 1;
    }
      
    if (IWANT[7] != 0)
    {
        LPHI = 1;
        LPHIT = 1;
    }
      
    if (IWANT[8] != 0) 
        LPHITT = 1;
      
    if (IWANT[9] != 0) 
    {
        LPHITT = 1;
        LPHID = 1;
        LPHIDD = 1;
        LPHIDT = 1;
    }
      
    if (IWANT[10] != 0) 
        LPHIT = 1;
      
    if (IWANT[11] != 0) 
        LPHI = 1;
      
    if (IWANT[12] != 0) 
    {
        LPHI = 1;
        LPHID = 1;
    }
      
    if (IWANT[13] != 0) 
    {
        LPHI = 1;
        LPHID = 1;
    }
      
    if ((IWANT[16] != 0) || (IWANT[14] != 0) || (IWANT[27] != 0)) 
    {
        LPHID = 1;
        LPHIDD = 1;
    }
      
    if (IWANT[15] != 0) 
    {
        LPHID = 1;
        LPHIDD = 1;
        LPHIDT = 1;
    }
      
    if (IWANT[17] != 0) 
    {
        LPHID = 1;
        LPHIDT = 1;
    }
      
    if (IWANT[18] != 0) 
    {
        LPHID = 1;
        LPHIDD = 1;
        LPHITT = 1;
        LPHIDT = 1;
    }
      
    if (IWANT[19] != 0) 
    {
        LPHID = 1;
        LPHIDD = 1;
        LPHITT = 1;
        LPHIDT = 1;
    }
      
    if (IWANT[23] != 0) 
    {
        LPHID = 1;
        LPHIDD = 1;
        LPHDDD = 1;
    }
      
    if (IWANT[24] != 0) 
        LPHDTT = 1;
      
    if (IWANT[25] != 0) 
    {
        LPHID = 1;
        LPHIDD = 1;
        LPHIDT = 1;
        LPHDDT = 1;
    }
      
    if (IWANT[26] != 0) 
    {
        LPHID = 1;
        LPHIDD = 1;
        LPHIDT = 1;
    }
      
    if (IWANT[28] != 0) 
    {
        LPHID = 1;
        LPHIDD = 1;
        LPHIDT = 1;
        LPHDDD = 1;
        LPHDDT = 1;
        LPHDTT = 1;
    }
      
    if (IWANT[29] != 0) 
    {
        LPHID = 1;
        LPHIDD = 1;
        LPHDDD = 1;
    }
      
    if (IWANT[30] != 0) 
    {
        LPHID = 1;
        LPHIDD = 1;
        LPHIDT = 1;
        LPHDDD = 1;
        LPHDDT = 1;
    }
      
/*
C NOW CALL THE IDEAL AND (UNLESS IGFLG=1) RESIDUAL HELMHOLTZ ENERGIES
*/
    AIDEAL(LPHI, LPHID, LPHIT, LPHIDD, LPHITT, LPHIDT, LPHDDD, LPHDDT, LPHDTT, 
           &PHI0, &PHID0, &PHIT0, &PHIDD0, &PHITT0, &PHIDT0, &PHDDD0, &PHDDT0, &PHDTT0, 
           TAU, DEL);
     
    if (IGFLG != 1) 
    {
    	ARESID(LPHI, LPHID, LPHIT, LPHIDD, LPHITT, LPHIDT, LPHDDD, LPHDDT, LPHDTT, 
               &PHIR, &PHIDR, &PHITR, &PHIDDR, &PHITTR, &PHIDTR, &PHDDDR, &PHDDTR, &PHDTTR,
               TAU, DEL);
    }
    else
    {
    	PHIR = 0.0;
        PHIDR = 0.0;
        PHIDDR = 0.0;
        PHITR = 0.0;
        PHITTR = 0.0;
        PHIDTR = 0.0;
        PHDDDR = 0.0;
        PHDDTR = 0.0;
        PHDTTR = 0.0;
    }
/*
C NOW PUT THE TERMS TOGETHER TO CALCULATE THE REQUESTED REDUCED PROPS
*/
    if (IWANT[2] != 0) 
        PROPR[2] = 1.0 + DEL*PHIDR;
      
    if (IWANT[6] != 0) 
        PROPR[6] = 1.0 + TAU*(PHIT0+PHITR) + DEL*PHIDR;
      
    if (IWANT[7] != 0) 
        PROPR[7] = TAU*(PHIT0+PHITR) - PHI0 - PHIR;
      
    if (IWANT[8] != 0) 
        PROPR[8] = -TAU*TAU*(PHITT0+PHITTR);
      
    if (IWANT[9] != 0) 
        PROPR[9] = -TAU*TAU*(PHITT0+PHITTR) + pow((1.0 + DEL*PHIDR-DEL*TAU*PHIDTR),2) /
                    (1.0 + 2.0*DEL*PHIDR+DEL*DEL*PHIDDR); 
      
    if (IWANT[10] != 0) 
        PROPR[10] = TAU*(PHIT0+PHITR);
      
    if (IWANT[11] != 0) 
        PROPR[11] = PHI0 + PHIR;
      
    if (IWANT[12] != 0) 
        PROPR[12] = PHI0 + PHIR + 1.0 + DEL*PHIDR;
      
    if (IWANT[13] != 0) 
    {
	/* ALSO RETURN PRESSURE SO WILL BE ABLE TO COMPUTE FUGACITY  */
        PROPR[2] = 1.0 + DEL*PHIDR;
        
        if (PROPR[2] > 0.0)
        {
          PROPR[13] = PHIR + (PROPR[2] - 1.0) - log(PROPR[2]);
          PROPR[13] = exp(PROPR[13]);
        }
        else
          PROPR[13] = 0.0;
    }
      
    if (IWANT[14] != 0) 
        PROPR[14] = 1.0 / (1.0 + 2.0*DEL*PHIDR + DEL*DEL*PHIDDR);
      
    if (IWANT[15] != 0) 
        PROPR[15] = (1.0 + DEL*PHIDR - TAU*DEL*PHIDTR) /
                    (1.0 + 2.0*DEL*PHIDR + DEL*DEL*PHIDDR);
      
    if (IWANT[16] != 0) 
        PROPR[16] = 1.0 + 2.0*DEL*PHIDR + DEL*DEL*PHIDDR;
      
    if (IWANT[17] != 0) 
        PROPR[17] = 1.0 + DEL*PHIDR - TAU*DEL*PHIDTR;
      
    if (IWANT[18] != 0) 
        PROPR[18] = 1.0 + 2.0*DEL*PHIDR + DEL*DEL*PHIDDR - pow((1.0 + DEL*PHIDR-DEL*TAU*PHIDTR),2) 
                  / (TAU*TAU*(PHITT0+PHITTR));
      
    if (IWANT[19] != 0) 
        PROPR[19] = -(DEL*PHIDR + DEL*DEL*PHIDDR + DEL*TAU*PHIDTR) / 
                     (pow((1.0 + DEL*PHIDR-DEL*TAU*PHIDTR),2) - TAU*TAU*(PHITT0 + PHITTR)*
                     (1.0 + 2.0*DEL*PHIDR+DEL*DEL*PHIDDR));
      
    if (IWANT[23] != 0) 
        PROPR[23] = DEL * (2.0*PHIDR + 4.0*DEL*PHIDDR + DEL*DEL*PHDDDR);
      
    if (IWANT[24] != 0) 
        PROPR[24] = DEL*TAU*TAU*PHDTTR;
      
    if (IWANT[25] != 0) 
        PROPR[25] = 1.0 + 2.0*DEL*PHIDR - 2.0*DEL*TAU*PHIDTR + DEL*DEL*(PHIDDR - TAU*PHDDTR);
       
/*
C FOR DERIVATIVES OF DENSITY, COMPUTE SOME QUANTITIES USED IN COMMON
*/
    if ((IWANT[26] != 0) || (IWANT[27] != 0) || (IWANT[28] != 0) || (IWANT[29] != 0) || (IWANT[30] != 0)) 
    	C = 1.0 + 2.0*DEL*PHIDR + DEL*DEL*PHIDDR;
      
    if ((IWANT[26] != 0) || (IWANT[28] != 0) || (IWANT[30] != 0))
    {
        B = 1.0 + DEL*PHIDR - DEL*TAU*PHIDTR;
        DRT = -B / C;
    }
      
    if ((IWANT[28] != 0) || (IWANT[30] != 0)) 
    {
        DCR = DEL * (2.0*PHIDR + 4.0*DEL*PHIDDR + DEL*DEL*PHDDDR);
        DCT = -DEL*TAU * (2.0*PHIDTR + DEL*PHDDTR);
    }
      
    if (IWANT[26] != 0) 
        PROPR[26] = DRT;
      
    if (IWANT[27] != 0) 
        PROPR[27] = 1.0 / C;
      
    if (IWANT[28] != 0) 
    {
        DBT = DEL*TAU*TAU*PHDTTR;
        DBR = DEL * (PHIDR + DEL*PHIDDR - TAU*PHIDTR - DEL*TAU*PHDDTR);
        DXT = (B - DBT + B*DCT/C) / C;
        DXR = -(B + DBR - B*DCR/C) / C;
        PROPR[28] = DXT + DXR*DRT;
    }
      
    if (IWANT[29] != 0) 
    {
        DRP = 1.0 / C;
        D2PDRR = DEL * (2.0*PHIDR + 4.0*DEL*PHIDDR + DEL*DEL*PHDDDR);
        PROPR[29] = -pow(DRP,3) * D2PDRR;
    }
      
    if (IWANT[30] != 0) 
    {
        DXT = -(1.0 + DCT/C) / C;
        DXR = -DCR / (C*C);
        PROPR[30] = DXT + DXR*DRT;
    }
    
    return;
}


void WTRANS(int MODE, double TK, double RHO, double CHI, double DPDT, double *THCOND, 
            double *VISCOS)
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** WTRANS ***                                   C
C THIS ROUTINE COMPUTES THE THERMAL CONDUCTIVITY AND VISCOSITY OF      C
C WATER AT A GIVEN TEMPERATURE AND DENSITY ACCORDING TO THE EQUATIONS  C
C PRESENTED BY KESTIN ET AL., JPCRD, 13, 175 (1984).                   C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C MODE    I    I   FOR WHETHER OR NOT TO COMPUTE VARIOUS PROPERTIES,   C
C                  1=THCOND ONLY, 2=VISC ONLY, 3=BOTH                  C
C TK      R    I   TEMPERATURE IN KELVINS                              C
C RHO     R    I   DENSITY IN KG/M3                                    C
C CHI     R    I   ISOTHERMAL COMPRESSIBILITY IN MPA**(-1)             C
C DPDT    R    I   DPDT IN MPA/K                                       C
C THCOND  R    O   THERMAL CONDUCTIVITY IN W/m.K                       C
C VISCOS  R    O   VISCOSITY IN PA.S                                   C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 25SEP95 - INITIAL CREATION BY AHH                                    C
C 12AUG96 - CORRECT CALCULATION OF CHIBAR                              C
C 18SEP96 - AHH: CONVERT TREF TO 1990 TEMPERATURE SCALE (WAS 647.27)   C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
	double TBAR, PBAR, RBAR, CHIBAR, VISC0, VISCBR, COND0, CONDBR, DPDTBR, COND1;
/*
C REFERENCE CONSTANTS FROM KESTIN ET AL.
*/
	static const double TREF = 647.226;
	static const double RHOREF = 317.763;
	static const double PREF = 22.115;
	static const double CNDREF = 0.4945;
	static const double VREF = 55.071e-6;

/*
C COMPUTE "REDUCED" TEMPERATURE AND DENSITY AND COMPR.
*/
    TBAR  = TK / TREF;
    RBAR = RHO / RHOREF;
    CHIBAR = CHI * PREF * RBAR * RBAR;
/*
C COMPUTE VISCOSITY ALWAYS, SINCE NEEDED FOR THCOND ALSO
*/
    VISC(TBAR, RBAR, CHIBAR, &VISC0, &VISCBR);
    *VISCOS = VISCBR*VREF;
/*
C COMPUTE THERMAL CONDUCTIVITY IF REQUESTED
*/
    if ((MODE == 1) || (MODE == 3)) 
    {
/*
C COMPUTE REDUCED DPDT, THEN CALL FOR BACKGROUND PART
*/
        DPDTBR = DPDT*TREF/PREF;
        COND0 = BKCOND (TBAR, RBAR);
/*
C CALL FOR ANOMALOUS PART OF CONDUCTIVITY UNLESS RHO IS HIGH
*/
        if (RHO/RHOREF < 3.0) 
        {
           COND1 = ANCOND(TBAR, RBAR, CHIBAR, DPDTBR, VISC0);
           CONDBR = COND0 + COND1;
        }
        else
           CONDBR = COND0;
        
        *THCOND = CONDBR*CNDREF;
    }
      
    return;
}


double BKCOND(double TBAR, double RBAR)
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** BKCOND ***                                   C
C THIS ROUTINE COMPUTES THE BACKGROUND PART OF THE THERMAL CONDUCTIVITYC
C OF WATER AT A GIVEN TEMPERATURE AND DENSITY ACCORDING TO THE EQNS    C
C PRESENTED BY KESTIN ET AL., JPCRD, 13, 175 (1984).                   C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C TBAR    R    I   TEMPERATURE MADE DIMENSIONLESS BY TSTAR             C
C RBAR    R    I   DENSITY MADE DIMENSIONLESS BY RSTAR                 C
C COND    R    O   BACKGROUND THERMAL CONDUCTIVITY (DIMENSIONLESS)     C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 25SEP95 - INITIAL ADAPTATION FROM OLD STEAM PROGRAM BY AHH           C
C 31JUL96 - AHH: CORRECT "A" ARRAY FOR PROPER USE OF REFERENCE CONST   C
C 11FEB97 - AHH: MAKE EXP'S UNIFORMLY DOUBLE PRECISION                 C
C 15MAY97 - AHH: MAKE COEFFICIENTS DOUBLE PRECISION                    C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
	double COND0, DR, TINV, DT, THCOND, SUM;
	
    static const double B[6][7] = {
    	{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
    	{0.0, 1.3293046, -.40452437, 0.24409490, 0.018660751, -.12961068, 0.044809953},
    	{0.0, 1.7018363, -2.2156845, 1.6511057,  -0.76736002, 0.37283344, -0.1120316},
    	{0.0, 5.2246158, -10.124111, 4.9874687,  -0.27297694, -.43083393, 0.13333849},
    	{0.0, 8.7127675, -9.5000611, 4.3786606,  -0.91783782, 0.0,        0.0},
    	{0.0, -1.8525999, 0.9340469, 0.0,        0.0,         0.0,        0.0}
    };
    
    static const double A0 = 1.0;
    static const double A1 = 6.978267;
    static const double A2 = 2.599096;
    static const double A3 = -0.998254;

    DR = RBAR - 1.0;
    TINV = 1.0 / TBAR;
    DT = TINV - 1.0;

    THCOND = A0 + TINV*(A1 + TINV*(A2 + A3*TINV));
    THCOND = sqrt(TBAR)/THCOND;
    
    SUM = 0.0;
      
    for (int i=1; i<=5; i++)
    {
    	for (int j=1; j<=6; j++)
   			SUM = SUM + B[i][j]*pow(DR,j-1)*pow(DT,i-1);
   	}
   
    COND0 = THCOND*exp(RBAR*SUM);
      
    return COND0;
}


double ANCOND(double TBAR, double RBAR, double CHIBAR, double DPDTBR, double VBAR0)
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** ANCOND ***                                   C
C THIS ROUTINE COMPUTES THE ANOMALOUS PART OF THE THERMAL CONDUCTIVITY C
C OF WATER AT A GIVEN TEMPERATURE AND DENSITY ACCORDING TO THE EQNS    C
C PRESENTED BY KESTIN ET AL., JPCRD, 13, 175 (1984).                   C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C TBAR    R    I   TEMPERATURE MADE DIMENSIONLESS BY TSTAR             C
C RBAR    R    I   DENSITY MADE DIMENSIONLESS BY RSTAR                 C
C CHIBAR  R    I   DIMENSIONLESS ISOTHERMAL COMPRESSIBILITY            C
C DPDTBR  R    I   DIMENSIONLESS DP/DT                                 C
C VBAR0   R    I   DIMENSIONLESS NORMAL PART OF VISCOSITY              C
C COND1   R    O   ANOMALOUS THERMAL CONDUCTIVITY (DIMENSIONLESS)      C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 25SEP95 - INITIAL ADAPTATION FROM OLD STEAM PROGRAM BY AHH           C
C 31JUL96 - AHH: STREAMLINE CODE                                       C
C 25SEP96 - AHH: PREVENT UNDERFLOW                                     C
C 11FEB97 - AHH: MAKE EXP'S UNIFORMLY DOUBLE PRECISION                 C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
	double COND1, DT, DR, CHIR, EX, FAC;
	
    static const double A = 18.66;
    static const double C = 1.3848e-03;
    static const double OMEGA = 0.4678;

    DT = TBAR - 1.0;
    DR = RBAR - 1.0;
      
    if (DPDTBR < 0.0) 
    	DPDTBR = 0.0;
      
    CHIR = pow(fabs(CHIBAR),OMEGA);
    EX = -A*DT*DT - pow(DR,4);
      
    if (EX < -600.0) 
        COND1 = 0.0;
    else
    {
        FAC = pow((TBAR/RBAR*DPDTBR),2) * CHIR * sqrt(RBAR);
        COND1 = C*FAC*exp(EX)/VBAR0;
    }
      
    return COND1;
}


void VISC(double TBAR, double RBAR, double CHIBAR, double *VISC0, double *VISCBR)
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** VISC ***                                     C
C THIS ROUTINE COMPUTES THE VISCOSITY                                  C
C OF WATER AT A GIVEN TEMPERATURE AND DENSITY ACCORDING TO THE EQNS    C
C PRESENTED BY KESTIN ET AL., JPCRD, 13, 175 (1984).                   C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C TBAR    R    I   TEMPERATURE MADE DIMENSIONLESS BY TSTAR             C
C RBAR    R    I   DENSITY MADE DIMENSIONLESS BY RSTAR                 C
C CHIBAR  R    I   DIMENSIONLESS ISOTHERMAL COMPRESSIBILITY            C
C VISC0   R    O   DIMENSIONLESS NORMAL PART OF VISCOSITY              C
C VISCBR  R    O   DIMENSIONLESS TOTAL VISCOSITY                       C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 25SEP95 - INITIAL ADAPTATION FROM OLD STEAM PROGRAM BY AHH           C
C 11FEB97 - AHH: MAKE EXP'S UNIFORMLY DOUBLE PRECISION                 C
C 15MAY97 - AHH: MAKE COEFFICIENTS DOUBLE PRECISION                    C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
	double TINV, DT1, DR, SUM, VISCX, DT1I, V2;
/*
	NOTE: dimensions of A & B increased by 1 due to differences in way arrays are treated in C
			and Fortran.  Also, order of elements in B converted from the "column major" format
			used in Fortran to that of C.
*/
    static const double A[5] = {0.0, 1.0, 0.978197, 0.579829, 0-.202354};
          
	static const double B[7][8] = {
		{0.0,      0.0,        0.0,       0.0,       0.0,       0.0,        0.0,         0.0},
		{0.0, 0.5132047, 0.2151778, -.2818107, 0.1778064, -.0417661,        0.0,         0.0},
		{0.0, 0.3205656, 0.7317883, -1.070786, 0.4605040,       0.0, -.01578386,         0.0},
		{0.0,       0.0,  1.241044, -1.263184, 0.2340379,       0.0,        0.0,         0.0},
		{0.0,       0.0,  1.476783,       0.0, -.4924179,  .1600435,        0.0, -.003629481},
		{0.0, -.7782567,       0.0,       0.0,       0.0,       0.0,        0.0,         0.0},
		{0.0, 0.1885447,       0.0,       0.0,       0.0,       0.0,        0.0,         0.0}
	};

    TINV = 1.0 / TBAR;
    DT1 = TINV - 1.0;
    DR = RBAR - 1.0;
    SUM = 0.0;
      
    for (int k=1; k<=4; k++)
 		SUM = SUM + A[k]*pow(TINV,(k-1)); 
      
    VISCX = sqrt(TBAR)/SUM;
    SUM = 0.0;
      
    for (int i=1; i<=6; i++)
    {
    	DT1I = pow(DT1,(i-1));
      
      	for (int j=1; j<=7; j++)
			SUM = SUM + B[i][j]*DT1I*pow(DR,(j-1));
	}
      
    *VISC0 = VISCX*exp(RBAR*SUM);

    if (TBAR < 0.997 || TBAR > 1.0082 || DR < -.245 || DR > 0.29 || CHIBAR < 21.93) 
        *VISCBR = *VISC0;
    else
    {
        V2 = 0.922*pow(CHIBAR,0.0263);
        *VISCBR = *VISC0*V2;
    }
    return;
}


int DIELEC(double TK, double RHO, double *DCON, double *DEPSDT, double *DEPSDR, 
           double *D2EDTT, double *D2EDRR, double *D2EDTR, int MODE)
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** DIELEC ***                                   C
C THIS ROUTINE COMPUTES THE STATIC DIELECTRIC CONSTANT                 C
C OF WATER AT A GIVEN TEMPERATURE AND REDUCED DENSITY                  C
C ACCORDING TO THE EQUATION OF FERNANDEZ ET AL                         C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C TK      R    I   TEMPERATURE IN KELVINS                              C
C RHO     R    I   DENSITY IN KG/M3                                    C
C DCON    R    O   STATIC DIELECTRIC CONSTANT                          C
C DEPSDT  R    O   DERIVATIVE OF DCON WRT TK AT CONSTANT RHO           C
C DEPSDR  R    O   DERIVATIVE OF DCON WRT RHO AT CONSTANT TK           C 
C D2EDTT  R    O   2ND DERIVATIVE OF DCON WRT TK AT CONSTANT RHO       C
C D2EDRR  R    O   2ND DERIVATIVE OF DCON WRT RHO AT CONSTANT TK       C
C D2EDTR  R    O   MIXED 2ND DERIVATIVE OF DCON WRT TO TK, RHO         C
C MODE    I    I   MODE: 0 = CALC DIELECTRIC CONSTANT ONLY             C
C                        1 = CALC DEPSDT AND DEPSDR ALSO               C
C                        2 = CALC 2ND DERIVATIVES ALSO                 C
C IERR    I    O   RETURN FLAG: 0 = SUCCESSFUL CALCULATION             C
C                               1 = T TOO LOW (BELOW 228), DCON        C
C                                   RETURNED AS ZERO                   C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 27NOV95 - INITIAL IMPLEMENTATION BY AHH                              C
C 07DEC95 - COEFFICIENTS REVISED FROM NEW FIT                          C
C 15DEC95 - AHH: DON'T CALCULATE IF TEMPERATURE TOO LOW                C
C 15FEB96 - AHH: PUT IN CALCULATION OF FIRST DERIVATIVES               C
C 30AUG96 - AHH: LOWERCASE INCLUDES FOR OTHER PLATFORMS                C
C 09JAN97 - AHH: PUT IN CALCULATION OF SECOND DERIVATIVES              C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
    static const double WALPHA = 1.636e-40; 
    static const double WMU = 6.138e-30;
    static const double BOLTZ = 1.380658e-23;
    static const double AV = 6.0221367e23;
     
    static const double AN[13] = {0.0, 0.978224486826, -.957771379375, .237511794148, .714692244396,
                                  -.298217036956, -.108863472196, .949327488264e-1, 
                                  -.980469816509e-2, .165167634970e-4, .937359795772e-4, 
                                  -.123179218720e-9, .196096504426e-2};
                                  
    static const int II[12] = {0, 1, 1, 1, 2, 3, 3, 4, 5, 6, 7, 10};
      
    static const double AJJ[12] = {0.0, 0.25, 1.0, 2.5, 1.5, 1.5, 2.5, 2.0, 2.0, 5.0, 0.5, 10.0};
	
	int IERR;
    double PI, EPS0, RHOR, TINVR, GFACT, TTHING, RMOL, A, B, C, CROOT, THING1, THING2;
	double GRSUM, GTSUM, RTERM, TTERM, DGDR, DGDT, DADR, DBDR, DADT, DCDR, DCDT, GTRSUM;
	double D2GDRR, D2GDTT, D2GDTR, D2ADTT, D2ADRR, D2ADTR, D2CDRR, D2CDTT, D2CDTR;

    IERR = 0;
    *DEPSDT = 0.0;
    *DEPSDR = 0.0;
    *D2EDTT = 0.0;
    *D2EDRR = 0.0;
    *D2EDTR = 0.0;
/*
C CHECK TO SEE IF TEMPERATURE IS TOO LOW TO CALCULATE
*/
    if (TK <= 228.0) 
    {
        IERR = 1;
        *DCON = 0.0;
        return IERR;
    }
/*
C FIGURE REDUCED DENSITY AND INVERSE TEMPERATURE VARIABLES
*/
    PI = M_PI;
    EPS0 = 1.0 / (4.0e-7*PI*pow(299792458.0,2));
    RHOR = RHO / RHOCW;
    TINVR = TCW / TK;
/*
C FIGURE G-FACTOR
*/
    GFACT = 1.0;
    
    for (int i=1; i<=11; i++)
        GFACT = GFACT + AN[i]*pow(RHOR,II[i])*pow(TINVR,AJJ[i]);
  
    TTHING = (TK/228.0 - 1.0);
    GFACT = GFACT + AN[12]*RHOR / pow(TTHING,1.2);
/*
C CONVERT TO MOLAR DENSITY (G-MOLES/M3) USING MOLAR MASS
*/
    RMOL = 1.0e3 * RHO / WMMASS;
/*
C FIGURE "A" AND "B" QUANTITIES
*/
    A = AV*WMU*WMU*RMOL*GFACT / (EPS0*BOLTZ*TK);
    B = AV*WALPHA*RMOL / (3.0*EPS0);
/*
C COMPUTE DIELECTRIC CONSTANT
*/
    C = 9.0 + 2.0*A + 18.0*B + A*A + 10.0*A*B + 9.0*B*B;
    CROOT = sqrt(C);
    THING1 = 1.0 + A + 5.0*B;
    THING2 = 1.0 / (4.0 - 4.0*B);
    *DCON = (THING1 + CROOT) * THING2;
/*
C COMPUTE FIRST DERIVATIVES IF REQUESTED
*/
    if (MODE > 0) 
    {
        GRSUM = 0.0;
        GTSUM = 0.0;
        
        for (int i=1; i<=11; i++)
        {
          	RTERM = AN[i]*pow(RHOR,II[i]);
          	TTERM = pow(TINVR,AJJ[i]);
          	GRSUM = GRSUM + II[i]*RTERM*TTERM;
          	GTSUM = GTSUM + AJJ[i]*RTERM*TTERM;
  		}
  
        DGDR = GRSUM/RHO + AN[12]/RHOCW/pow(TTHING,1.2);
        DGDT = -GTSUM/TK - (1.20/228.0)*AN[12]*RHOR/pow(TTHING,2.2);
        DADR = A/RHO + A*DGDR/GFACT;
        DBDR = B / RHO;
        DADT = -A/TK + A*DGDT/GFACT;
        DCDR = 2.0*DADR*THING1 + 2.0*DBDR*(9.0 + 5.0*A + 9.0*B);
        DCDT = 2.0*DADT * THING1;
        *DEPSDT = THING2 * (DADT + 0.5*DCDT/CROOT); 
        *DEPSDR = (DADR + 5.0*DBDR + 0.5*DCDR/CROOT + 4.0* *DCON * DBDR) * THING2;
    }
/*
C COMPUTE SECOND DERIVATIVES IF REQUESTED
*/
    if (MODE > 1) 
    {
        GRSUM = 0.0;
        GTSUM = 0.0;
        GTRSUM = 0.0;
        
        for (int i=1; i<=11; i++)
        {
          RTERM = AN[i]*pow(RHOR,II[i]);
          TTERM = pow(TINVR,AJJ[i]);
          GRSUM = GRSUM + (II[i]*II[i]-II[i])*RTERM*TTERM;
          GTSUM = GTSUM + (AJJ[i]*AJJ[i]+AJJ[i])*RTERM*TTERM;
          GTRSUM = GTRSUM + II[i]*AJJ[i]*RTERM*TTERM;
  		}
  		
        D2GDRR = GRSUM / (RHO*RHO);
        D2GDTT = GTSUM/(TK*TK) + (2.64/pow(228.0,2)) * AN[12]*RHOR/pow(TTHING,3.2);
        D2GDTR = -GTRSUM/(RHO*TK) - 1.2*AN[12]/(228.0*RHOCW*pow(TTHING,2.2));
        
        D2ADRR = 2.0*A*DGDR/(RHO*GFACT) + A*D2GDRR/GFACT;
        D2ADTT = 2.0*A/(TK*TK) - 2.0*A*DGDT/(GFACT*TK) + A*D2GDTT/GFACT;
        D2ADTR = A*DGDT/(RHO*GFACT) - A/(RHO*TK) + A*D2GDTR/GFACT - A*DGDR/(GFACT*TK);
     
        D2CDRR = 2.0 * (D2ADRR*THING1 + DADR*DADR + 9.0*DBDR*DBDR + 10.0*DADR*DBDR);
        D2CDTT = 2.0 * (D2ADTT*THING1 + DADT*DADT);
        D2CDTR = 2.0 * (D2ADTR*THING1 + DADT*(DADR + 5.0*DBDR));
        
        *D2EDTT = THING2 * (D2ADTT + 0.5*(D2CDTT - 0.5*DCDT*DCDT/C)/CROOT);
        *D2EDRR = THING2 * (D2ADRR + 8.0*DBDR* (*DEPSDR) + 0.5*(D2CDRR - 0.5*DCDR*DCDR/C)/CROOT);
        *D2EDTR = THING2 * (D2ADTR + 4.0*DBDR* (*DEPSDT) + 0.5*(D2CDTR - 0.5*DCDT*DCDR/C)/CROOT);
    }
    
    return IERR;
}


void RIND(double TK, double RHO, double ALAM[], double RINDX[], int IERLAM[], int N)
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** RIND  ***                                    C
C THIS SUBROUTINE COMPUTES THE REFRACTIVE INDEX OF WATER AT A          C
C GIVEN TEMPERATURE, DENSITY, AND ARRAY OF WAVELENGTHS                 C
C ACCORDING TO THE REFIT EQUATION OF SCHIEBENER ET AL                  C
C IT TAKES AN ARRAY OF WAVELENGTHS AND CALLS RIND2 FOR EACH OF THEM    C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C TK      R    I   TEMPERATURE IN KELVINS                              C
C RHO     R    I   DENSITY IN KG/M3                                    C
C ALAM    RA   I   WAVELENGTHS (MICROMETERS) AT WHICH TO COMPUTE RINDX C
C RINDX   RA   O   REFRACTIVE INDEX AT ALAM VALUES                     C
C IERLAM  IA   O   ARRAY OF RETURN FLAGS FOR WAVELENGTHS               C
C                  0     = OK                                          C
C                  1(-1) = LARGER(SMALLER) THAN RECOMMENDED RANGE,     C
C                          VALUE IS AN EXTRAPOLATION                   C
C                  2(-2) = TOO LARGE(SMALL), BEYOND REASONABLE         C
C                          EXTRAPOLATION RANGE. WILL STILL RETURN      C
C                          COMPUTED NUMBER IF WITHIN RANGE WHERE CALC. C
C                          DOESN'T BLOW UP, OTHERWISE RETURNS ZERO.    C
C N       I    I   LENGTH OF ALAM, RINDX, AND IERLAM ARRAYS            C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 19AUG96 - INITIAL IMPLEMENTATION BY AHH                              C
C 25MAR97 - AHH: CHANGES FOR CALLING UNDER PROPS0                      C
C 27MAY97 - AHH: CHANGE ERROR/WARNING HANDLING                         C
C 22SEP97 - AHH: ADJUST UPPER EXTRAPOLATION LIMIT TO 1.9 IN ACCORDANCE C
C                WITH IAPWS RELEASE                                    C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
    static const double TREF = 273.15; 
    static const double RREF = 1.0e3;
    static const double ALREF = 0.589;
    static const double ABIG = 1.10;
    static const double ASMALL = 0.2; 
    static const double BIG2 = 1.90; 
    static const double SMALL2 = 0.18;

	double TBAR, RBAR, ALAMBR;
	
    TBAR = TK / TREF;
    RBAR = RHO / RREF;
/*
C - LOOP THROUGH EACH POINT, CALLING RIND2 FOR CALCULATION
*/
    for (int i=1; i<=N; i++)
    {
        ALAMBR = ALAM[i] / ALREF;
        RINDX[i] = RIND2(ALAMBR, RBAR, TBAR);
/*
C - CHECK TO SEE IF OUTSIDE RECOMMENDED BOUNDS
*/
        if (ALAM[i] < SMALL2) 
          IERLAM[i] = -2;
          
        else if (ALAM[i] > BIG2) 
          IERLAM[i] = 2;
          
        else if (ALAM[i] < ASMALL) 
          IERLAM[i] = -1;
          
        else if (ALAM[i] > ABIG) 
          IERLAM[i] = 1;
          
        else
          IERLAM[i] = 0;
  	}

    return;
}


double RIND2(double ALAMBR, double RBAR, double TBAR)
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** RIND2 ***                                    C
C THIS SUBROUTINE COMPUTES THE REFRACTIVE INDEX OF WATER AT A          C
C GIVEN REDUCED TEMPERATURE, REDUCED DENSITY, AND REDUCED WAVELENGTH   C
C ACCORDING TO THE REFIT EQUATION OF SCHIEBENER ET AL                  C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C ALAMBR  R    I   REDUCED WAVELENGTH                                  C
C RBAR    R    I   REDUCED DENSITY                                     C
C TBAR    R    I   REDUCED TEMPERATURE                                 C
C RINDX   R    O   REFRACTIVE INDEX                                    C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 19AUG96 - INITIAL IMPLEMENTATION BY AHH                              C
C 27MAY97 - AHH: CHANGE ERROR/WARNING HANDLING                         C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
	double RINDX, ALAM2, RHS, RRHS;
	
    static const double ALAMUV=.229202; 
    static const double ALAMIR=5.432937;
    static const double A0=.244257733; 
    static const double A1=9.74634476e-3;
    static const double A2=-3.73234996e-3;
    static const double A3=2.68678472e-4; 
    static const double A4=1.58920570e-3; 
    static const double A5=2.45934259e-3;
    static const double A6=.900704920; 
    static const double A7=-1.66626219e-2;

    if ((ALAMBR <= ALAMUV) || (ALAMBR >= ALAMIR)) 
        RINDX = 0.0;
    
    else
    {
        ALAM2 = ALAMBR*ALAMBR;
        RHS = A0 + A1*RBAR + A2*TBAR + A3*ALAM2*TBAR + A4/ALAM2 + A5/(ALAM2-ALAMUV*ALAMUV) 
            + A6/(ALAM2-ALAMIR*ALAMIR) + A7*RBAR*RBAR;
        RRHS = RBAR*RHS;
        RINDX = sqrt((2.0*RRHS + 1.0) / (1.0 - RRHS));
    }
      
    return RINDX;
}
